#include <PSX/GPU/gpu.hpp>
#include <PSX/GPU/gpu_primitives.hpp>
#include <FontWriter/font_writer.hpp>
#include <stdio.h>

#include <PSX/Auxiliary/mem_dump.hpp>

namespace JabyEngine {
    struct DoubleBuffer {
        GPU::TexPage::Linked tex_page;
        GPU::SPRT::Linked*   cur_sprt_ptr;
        FontBufferInfo       prim_buffer;

        GPU::SPRT_16::Linked planschi;

        static constexpr DoubleBuffer empty() {
            return DoubleBuffer{.tex_page = {0}, .cur_sprt_ptr = nullptr, .prim_buffer = FontBufferInfo::empty()};
        }

        void setup(const FontBufferInfo& buffer_info, const SimpleTIM& vram_dst, const GPU::SizeI16& font_size) {
            this->tex_page    = Make::TexPage(vram_dst.get_texture_position(), GPU::TexturePageColor::$4bit).linked();
            this->prim_buffer = buffer_info;

            this->planschi = Make::SPRT_16(
                Make::PositionI16(0, 0),
                Make::OffsetPageWithClut(Make::PageOffset(0, 0), Make::PageClut(vram_dst.get_clut_position()))
            ).linked();

            for(size_t buffer_id = 0; buffer_id < 2; buffer_id++) {
                for(size_t buffer_element_id = 0; buffer_element_id < buffer_info.single_buffer_length; buffer_element_id++) {
                    this->prim_buffer.double_buffer[buffer_id][buffer_element_id] = Make::SPRT(
                        Make::AreaI16(Make::PositionI16(0, 0), font_size),
                        Make::OffsetPageWithClut(Make::PageOffset(0, 0), Make::PageClut(vram_dst.get_clut_position()))
                    ).linked();

                    this->prim_buffer.double_buffer[buffer_id][buffer_element_id]->set_identitiy();
                }
            }
        }
    };

    static auto double_buffer = DoubleBuffer::empty();

    void FontWriter :: setup(const FontBufferInfo& buffer_info, const SimpleTIM& vram_dst, const FontInfo& font_info) {
        printf("Hello Planschi c: @0x%p @0x%p\n", buffer_info.double_buffer[0], buffer_info.double_buffer[1]);
        double_buffer.setup(buffer_info, vram_dst, Make::SizeI16(font_info.FontSize.width, font_info.FontSize.height));

        //TMP
            double_buffer.prim_buffer.double_buffer[0][0].terminate();
            double_buffer.tex_page.concat(double_buffer.prim_buffer.double_buffer[0][0]);
        //TMP-END
    }

    void FontWriter :: render() {
        GPU::render(double_buffer.tex_page);
    }
}